import logging
import os


class Base(object):
    """
    Handles common functionality across all cli command classes.
    """

    @staticmethod
    def _add_logging_options(argument_parser):
        """
        Add logging cli options.

        :param argparse.ArgumentParser argument_parser:
        :return:
        """

        logging_args = argument_parser.add_argument_group('Logging')
        logging_args.add_argument('--log-level', choices=['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'],
                                  default='ERROR')

    def __init__(self, logging_level, enable_logging=True, dry_run=False):
        """
        Specify the logging_level, and wether or not to perform actions

        :param string logging_level:
        :param bool dry_run:
        :return:
        """
        self.logging_level = logging_level
        self.enable_logging = enable_logging
        self.dry_run = dry_run
        self._init_logger()

    def _init_logger(self):
        """
        Initialises a logger
        
        :return:
        """
        self.logger = logging.getLogger(__name__)

        self.debug = (os.environ.get('DEBUG', '0') == '1')

        if self.debug:
            formatter = logging.Formatter(
                'AWS-RDSMan[%(threadName)10s:%(relativeCreated)5d:%(filename)17s:%(lineno)3s] %(levelname)s:%(message)s')
            stdout_handler = logging.StreamHandler()
            stdout_handler.setFormatter(formatter)
            self.logger.addHandler(stdout_handler)
            self.logger.setLevel(logging.DEBUG)
            self.logger.propagate = False
        else:
            formatter = logging.Formatter(
                'AWS-RDSMan: %(levelname)s:%(message)s'
            )
            stdout_handler = logging.StreamHandler()
            stdout_handler.setFormatter(formatter)
            self.logger.addHandler(stdout_handler)
            self.logger.setLevel(self.logging_level)
            self.logger.propagate = False

        if not self.enable_logging:
            self.logger.disabled = True



import json


def json_pprint(data):
    """
    Return the data as a string formatted json payload.

    :param object data:
    :return str: JSON encoded representation of the data.
    """
    print(json.dumps(
        data,
        sort_keys=True,
        indent=4,
        separators=(',', ': ')
    ))



import sys

from awsrdsmanager import json_pprint
from awsrdsmanager.sunsetrdssnapshots import SunsetRDSSnapshots


def sunset_rds_snapshots(args=sys.argv):
    """
    Wrapper for cli entrypoint for aws-rds-snapshot-cleanup

    :param List[str] args:
    :return:
    """
    json_pprint(
        SunsetRDSSnapshots.load(args).run()
    )



import argparse
import awsauthhelper

from awsrdsmanager.base import Base


class SunsetRDSSnapshots(Base):
    """
    Control the number of snapshots for an RDS database to be retained.
    """

    @classmethod
    def load(cls, args):
        """
        Parse cli arguments, and initialise the command

        :param List[str] args: List of cli args
        :return awsrdsmanager.SunsetRDSSnapshots:
        """

        awsargparser = cls._build_arg_parse()
        cli_options = awsargparser.parse_args(args=args[1:])

        # Perform authentication
        credentials = awsauthhelper.Credentials(**vars(cli_options))
        if credentials.has_role():
            credentials.assume_role()

        # Create a class
        return cls(
            database_name=cli_options.database_name,
            sunset_period=cli_options.sunset_period,
            dry_run=cli_options.dry_run,
            region=cli_options.region,
            logging_level=cli_options.log_level,
            session=credentials.create_session()
        )

    @classmethod
    def _build_arg_parse(cls):
        """
        Build the argument parser and return it

        :return:
        """
        argparser = argparse.ArgumentParser()
        argparser.add_argument('--database-name', help='Name of the database snapshot.', required=True)
        argparser.add_argument('--sunset-period', help='Number of snapshots to retains.', required=True, type=int)
        argparser.add_argument('--dry-run', help='Show snapshots to be deleted without deleting them',
                               action='store_true', default=False)

        cls._add_logging_options(argparser)

        awsargparser = awsauthhelper.AWSArgumentParser(role_session_name='register-instance', region='us-east-1',
                                                       parents=[argparser])
        return awsargparser

    def __init__(self, database_name, sunset_period, region, session, logging_level='INFO', enable_logging=True,
                 dry_run=False):
        """
        Set the region, session, and the database_name to operate on

        :param str database_name: Name of the database
        :param int sunset_period: How many snapshots to keep
        :param string region: Region to run this utility in
        :param callable(region=...) session: Function to return a botocore.Session object, with an optional region.
        :param string logging_level: Logging level
        :param bool enable_logging: True if logging should be enabled, and false if logging should be disabled.
        :param bool dry_run: True if no actions should be taken.
        :return:
        """
        super(SunsetRDSSnapshots, self).__init__(
            logging_level=logging_level,
            dry_run=dry_run,
            enable_logging=enable_logging
        )
        self.database_name = database_name
        self.sunset_period = sunset_period
        self.region = region
        self.session = session

    def run(self):
        """
        Remove any snapshots for the provided database_name which have more than the allowed number of snapshots.

        :return Dict[str, Dict[str, str]]:
        """

        self.rds_client = self._get_rds_client(region=self.region)

        snapshot_response = self.rds_client.describe_db_snapshots(
            DBInstanceIdentifier=self.database_name,
            SnapshotType='manual'
        )

        self.logger.debug('Snapshots : {0}'.format(
            snapshot_response['DBSnapshots']
        ))

        if not snapshot_response['DBSnapshots']:
            raise NoSnapshotsError(self.database_name)

        remaining_snapshots, deprecated_snapshots = self._sort_snapshots_by_threshold_period(
            self.sunset_period,
            snapshot_response['DBSnapshots']
        )

        if not self.dry_run:
            for snapshot in deprecated_snapshots:
                self.rds_client.delete_db_snapshot(DBSnapshotIdentifier=snapshot['DBSnapshotIdentifier'])

        return {
            'Snapshots': remaining_snapshots,
            'DeletedSnapshots': deprecated_snapshots
        }

    def _get_rds_client(self, region):
        """
        Return a boto3.rds client object

        :return:
        """
        return self.session(region=region).client('rds')

    def _sort_snapshots_by_threshold_period(self, threshold_count, snapshots):
        """
        Return 2 lists where the first is snapshots after the threshold, and the second is snapshots after the threshold

        :param Dict[str, str|int|datetime|bool] snapshots:
        :param int threshold_count: maximum number of snapshots to be returned in snapshots_after.
        :return snapshots_before, snapshots_after:
        """

        snapshots = filter(lambda snapshot: 'SnapshotCreateTime' in snapshot, snapshots)

        sorted_snapshots = self._sort_by_key('SnapshotCreateTime', snapshots)

        return sorted_snapshots[:threshold_count], sorted_snapshots[threshold_count:]

    @staticmethod
    def _sort_by_key(key, dict_list):
        """
        Sort a list of dictionaries by a key

        :param str|int key: Key to sort list by
        :param List[Dict[str|int, object]] dict_list: List of dictionaries to sort
        :return List[Dict[str|int, object]]: Sorted List of dictionaries
        """

        def sorter_callback(item):
            return item[key]

        return sorted(dict_list, key=sorter_callback, reverse=True)


class NoSnapshotsError(Exception):
    """
    Exception thrown if no snapshots are found
    """

    def __init__(self, database_name):
        super(NoSnapshotsError, self).__init__(
            "No snapshots found for database \'{name}\'".format(name=database_name)
        )



from setuptools import find_packages, setup

__version__ = '1.0.3'

setup(
    name='aws-rds-manager',
    description='CLI tool providing sunsetting of RDS snapshots',
    long_description=open('README.rst').read(),
    version=__version__,
    packages=['awsrdsmanager'],
    download_url='https://github.com/drewsonne/aws-rds-manager/archive/v.{version}.tar.gz'.format(version=__version__),
    install_requires=['aws-auth-helper', 'six'],
    url='',
    license='GPLv2',
    test_suite='tests',
    author='Drew J. Sonne',
    author_email='drew.sonne@gmail.com',
    entry_points={
        'console_scripts': [
            'aws-rds-snapshot-cleanup=awsrdsmanager.__main__:sunset_rds_snapshots'
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Database',
        'Topic :: System :: Archiving :: Backup'
    ]

)
